using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Runtime.Serialization;

namespace Actimizer.LRM.Contracts
{
    [DataContract]
    public class BaseCall
    {
        [Description( "A unique id for this call" )]
        [DataMember( IsRequired = true )]
        public string UniqueId { get; set; }

        [Description( "A unique lead id for this call" )]
        [DataMember( IsRequired = true )]
        public string UniqueLeadId { get; set; }

        [DataMember( IsRequired = true )]
        public string LeadPhoneNumber { get; set; }

        [Description( "The lead data field marked as external key" )]
        [DataMember]
        public string LeadExternalKey { get; set; }

        [DataMember( IsRequired = true )]
        public CallEndCause EndCause { get; set; }

        [Description( "True if the call was in progress at the time of object construction" )]
        [DataMember( IsRequired = true )]
        public bool InProgress { get; set; }

        [Description( "True if the call ever resulted in a connected event. This is _not_ an indication of whether the call is currently connected" )]
        [DataMember( IsRequired = true )]
        public bool Connected { get; set; }

        [Description( "True if the call started as an incoming call" )]
        [DataMember( IsRequired = true )]
        public bool Incoming { get; set; }

        [Description( "Indicates when dialing started or when incoming call arrived" )]
        [DataMember( IsRequired = true )]
        public DateTime StartTime { get; set; }

        [Description( "Time of connection. Only available if 'Connected' is true" )]
        [DataMember]
        public DateTime? ConnectTime { get; set; }

        [Description( "End time of the call. Only available when not in progress" )]
        [DataMember]
        public DateTime? EndTime { get; set; }

        [Description( "Preview duration occurs before StartTime" )]
        [DataMember( IsRequired = true )]
        public TimeSpan PreviewDuration { get; set; }

        [Description( "Not available during dialing" )]
        [DataMember]
        public TimeSpan? DialingDuration { get; set; }

        [Description( "Only available when Connected is true and not in progress" )]
        [DataMember]
        public TimeSpan? ConversationDuration { get; set; }

        [Description( "Only available after wrap up or after abandoned timeout" )]
        [DataMember]
        public TimeSpan? WrapUpDuration { get; set; }

        [Description( "The status set on the lead at wrap up. Only available after wrap up or after abandoned timeout. If the status on the lead is set to Unresolved then this property is set to Abandoned." )]
        [DataMember]
        public CallStatus? LeadStatus { get; set; }

        [Description("The status of the lead before the call was initiated.")]
        [DataMember]
        public PreCallStatus PreCallLeadStatus { get; set; }

        [Description( "The closure set on the lead at wrap up. Only available if LeadStatus = UserProcessed" )]
        [DataMember]
        public LeadClosure? LeadClosure { get; set; }

        [Description( "The type of the campaign which the called lead belongs to" )]
        [DataMember]
        public CampaignType? CampaignType { get; set; }
    }



    [DataContract]
    public class Call : BaseCall
    {
        [Description( "The campaign which the called lead belongs to" )]
        [DataMember( IsRequired = true )]
        public Campaign Campaign { get; set; }

        [Description( "The user making the call" )]
        [DataMember( IsRequired = true )]
        public User User { get; set; }

        [Description( "The organizational unit ancestors of the user making the call" )]
        [DataMember( IsRequired = true )]
        public IList<OrgUnit> OrganizationalUnits { get; set; }
    }



    /// <summary>
    /// This flattened call structure is for when calls should be serialized to a format,
    /// which does not support nested data, such as csv.
    /// I contains the same elements as the normal call structure, the difference being
    /// that the nested properties of Campaign, User and OrganizationalUnits, have been
    /// extracted into this flat structure.
    /// This object shouldn't be created directly, but rather instantiated via the
    /// AutoMapper mapping.
    /// The naming scheme used for the flattening is {PropertyName}{NestedPropertyName},
    /// so that the Code property on the Campaign class property, is called CampaingCode,
    /// and the Username property on the User class property is called UserUsername. This
    /// naming scheme is what AutoMapper uses for auto flattening, so using it reduces
    /// the need for explicit mapping.
    /// </summary>
    [DataContract]
    public class FlattenedCall : BaseCall
    {
        // The following properties have been copied from the Campaign class
        [Description( "The code of the campaign which the called lead belongs to" )]
        [DataMember( IsRequired = true )]
        public string CampaignCode { get; set; }

        [Description( "The name of the campaign which the called lead belongs to" )]
        [DataMember]
        public string CampaignName { get; set; }

        [Description( "The project of the campaign which the called lead belongs to" )]
        [DataMember]
        public string CampaignProject { get; set; }

        // The following properties have been copied from the User class
        [Description( "The orgcode of the user making the call" )]
        [DataMember]
        public string UserOrgCode { get; set; }

        [Description( "The username of the user making the call" )]
        [DataMember]
        public string UserUsername { get; set; }

        [Description( "The name of the user making the call" )]
        [DataMember]
        public string UserName { get; set; }

        [Description( "The level 0 (root) organizational unit of the user making the call" )]
        [DataMember( IsRequired = true )]
        public string OrganizationalUnit0OrgCode { get; set; }

        [Description( "The level 1 organizational unit ancestor of the user making the call" )]
        [DataMember]
        public string OrganizationalUnit1OrgCode { get; set; }

        [Description( "The level 2 organizational unit ancestor of the user making the call" )]
        [DataMember]
        public string OrganizationalUnit2OrgCode { get; set; }

        [Description( "The level 3 organizational unit ancestor of the user making the call" )]
        [DataMember]
        public string OrganizationalUnit3OrgCode { get; set; }

        [Description( "The level 4 organizational unit ancestor of the user making the call" )]
        [DataMember]
        public string OrganizationalUnit4OrgCode { get; set; }
    }
}
