namespace Actimizer.LRM.Contracts
{
	/// <summary>
	/// This enum contains the statuses that can be set on calls.
	/// A calls status is basically set to whatever the leads status is set to
	/// during wrap up, so most of these are similar to the statuses which can 
	/// be set on leads.
	/// There are a few lead statuses which can not be set as the result of a 
	/// call, so they are not included here.
	/// Since a call might end up not resulting in a lead status change, there
	/// are special call statuses for those cases.
	/// </summary>
    public enum CallStatus
    {
		/// <summary>
		/// No status set. Usually for call in progress, otherwise might be the
		/// result of an error.
		/// </summary>
        NotSet = 0,

		/// <summary>
		/// The lead status was set to automatic redial after this call. 
		/// Probable causes are: NoAnswer, Busy, VoiceMail.
		/// </summary>
        RedialAutomatic = 1,

		/// <summary>
		/// The lead status was set to common redial after this call.
		/// </summary>
        RedialManualCommon = 2,

		/// <summary>
		/// The lead status was set to private redial after this call.
		/// </summary>
        RedialManualPrivate = 3,

        /// <summary>
        /// This status is set when an agent schedules a redial and makes it
        /// VIP.
        /// </summary>
        VIPRedial = 10,

        /// <summary>
        /// This status is set by life cycle events. Leads with this status
        /// will be selected for dialing with higher priority than VIP.
        /// </summary>
        FollowUpRedial = 20,

        /// <summary>
        /// This call was succeeded by a call to the next number found on the lead and so
        /// didn't result in a lead status change. This status is unique to calls.
        /// </summary>
        NextNumberDialed = 198,

        /// <summary>
        /// This call was succeeded by a call to an alternative number and so,
        /// didn't result in a lead status change. This status is unique to calls.
        /// </summary>
        AlternativeNumberDialed = 199,

		/// <summary>
		/// The lead status was set to user processed after this call.
		/// </summary>
        UserProcessed = 200,

		/// <summary>
		/// The lead status was set to depleted after this call.
		/// </summary>
        Depleted = 210,

		/// <summary>
		/// When a lead call is transferred it is marked as disconnected and the 
		/// status is set as usual. To keep track of what happens after the 
		/// transfer, new calls are created and they are always set to status
		/// Transferred. This status is unique to calls.
		/// </summary>
        Transferred = 220,

        ThirdPartyCall = 230,

        /// <summary>
		/// When an agent A makes internal third party call
        /// to another agent B then a new lead call will be created
        /// for agent B with pre call status Internal. This status is unique to calls.
		/// </summary>
        InternalCall = 240,

        /// <summary>
        /// If an agent closes the contact page without saving the lead, the 
        /// lead status is set to Unresolved and if the lead is not later
        /// edited and set to a proper status it is eventually set to Abandoned.
        /// The Unresolved status is not used for calls, instead the call 
        /// status is set to Abandoned.
        /// </summary>
        Abandoned = 300,

		/// <summary>
		/// If the system detects that a lead is unreachable for other reasons
		/// than the lead phone never being answered, the lead status is set to
		/// Failed.
		/// </summary>
		Failed = 400,
    }

}
