using System.Collections.Generic;
using System.Linq;

namespace Actimizer.LRM.Contracts
{
    /// <summary>
    /// This enum contains the statuses that can be set on leads.
    /// If a lead changes status as the result of a call, the resulting status
    /// is also registered on the call.
    /// Some statuses cannot happen as the result of a call and are therefore
    /// unique to leads.
    /// </summary>
    public enum LeadStatus
    {
        /// <summary>
        /// New lead. Never before dialed.
        /// </summary>
        New = 0,

        /// <summary>
        /// This status is set if a call ended with NoResponse, Busy, VoiceMail.
        /// This status is only set if the previous lead state was New or
        /// RedialAutomatic. Common and private redials are never set to this
        /// state.
        /// </summary>
        RedialAutomatic = 1,

        /// <summary>
        /// This status is set when an agent schedules a redial and makes it
        /// available to all agents.
        /// </summary>
        RedialManualCommon = 2,

        /// <summary>
        /// This status is set when an agent schedules a redial and makes it
        /// private.
        /// </summary>
        RedialManualPrivate = 3,

        /// <summary>
        /// This status is set when an agent schedules a redial and makes it
        /// VIP.
        /// </summary>
        VIPRedial = 10,

        /// <summary>
        /// This status is set by life cycle events. Leads with this status
        /// will be selected for dialing with higher priority than VIP.
        /// </summary>
        FollowUpRedial = 20,



        /// <summary>
        /// These statuses are set by life cycle events. Leads with these statuses
        /// are in a waiting state and will not be dialed, but are not
        /// considered closed.
        /// </summary>

        // NOTE: When adding more life cycle statuses, make sure to add them in IsLifeCycle further down.

        // Simply a marker status indicating that contact page handling is done and that first processed time should be set
        // Will likely only ever be just as a transition to some other life cycle status.
        LifeCycleInitiation = 101,

        ValidationAwaiting = 105,
        ValidationRejected = 106,
        ValidationApproved = 107,

        AwaitingExternalEvents = 110,
        //AwaitingExternalEvents2 = 111,
        //AwaitingExternalEvents3 = 112,
        //AwaitingExternalEvents4 = 113,

        OrderConfirmationReadyFor = 115,
        OrderConfirmationAwaiting = 116,
        OrderConfirmationFailed = 117,
        OrderConfirmationExpired = 118,
        OrderConfirmationApproved = 119,

        PaymentReadyFor = 120,
        PaymentAwaiting = 121,
        PaymentFailed = 122,
        PaymentExpired = 123,
        PaymentApproved = 124,

        MeetingFeedbackAwaiting = 125,

        /// <summary>
        /// This status is set when an agent doesn't save a called lead on the
        /// contact page. The state is not set immediately, but rather next
        /// time the agent tries to call a lead, on logout, on session timeout
        /// or during nightly cleanup cycle. This status is unique to leads.
        /// </summary>
        Unresolved = 150,

        /// <summary>
        /// This status is set when the agent closes lead normally.
        /// </summary>
        UserProcessed = 200,

        /// <summary>
        /// If the lead has been called a given amount of times, but with no
        /// contact, the lead is set to Depleted.
        /// </summary>
        Depleted = 210,

        /// <summary>
        /// If an Unresolved lead hasn't been resolved within a given amount of
        /// time it is set to Abandoned.
        /// </summary>
        Abandoned = 300,

        /// <summary>
        /// If the system detects that a lead is unreachable for other reasons
        /// than the lead phone never being answered, the lead status is set to
        /// Failed.
        /// </summary>
        Failed = 400,

        /// <summary>
        /// If a lead hasn't been closed within a given time limit, set on the
        /// campaign, it is set to Expired.
        /// If no time limit is set on the campaign, its leads are never expired.
        /// New, auto and manual redials are expired according to the same
        /// setting. Private redials are expired according to a separate setting.
        /// VIP redials are never expired.
        /// </summary>
        Expired = 500,

        /// <summary>
        /// This status can be set by administrators to signal that the lead
        /// should no longer be used. This status is unique to leads.
        /// </summary>
        Removed = 600,

        /// <summary>
        /// User request to clear info about him, all lead data is cleared from the system according
        /// </summary>
        Anonymized = 610,

        /// <summary>
        /// This status can be set by administrators to signal that the lead
        /// should not be called, since it in customers DoNotCall list.
        /// </summary>
        DoNotCall = 700,

        /// <summary>
        /// This status can be set by superadministrators to signal that the lead
        /// should not be called, since it in Global DoNotCall list.
        /// </summary>
        GlobalDoNotCall = 750,
    }

    public enum LeadImportStatus
    {
        New = 0,
        RedialManualPrivate = 3,
        VIPRedial = 10
    }

    public static class LeadStatuses {
        public static IReadOnlyList<LeadStatus> OpenPrivate { get; } =
            new[] { LeadStatus.RedialManualPrivate, LeadStatus.VIPRedial, LeadStatus.FollowUpRedial };


        public static IReadOnlyList<LeadStatus> UserProcessedOrLifeCycle { get; } =
            System.Enum.GetValues( typeof( LeadStatus ) ).Cast<LeadStatus>().Where( LeadStatusExtensions.IsUserProcessedOrLifeCycle ).ToList();
        //new[] { LeadStatus.RedialManualPrivate, LeadStatus.VIPRedial, LeadStatus.FollowUpRedial };

        public static IReadOnlyList<LeadStatus> OpenOrLifeCycle { get; } =
            System.Enum.GetValues( typeof( LeadStatus ) ).Cast<LeadStatus>( ).Where( LeadStatusExtensions.IsOpenOrLifeCycle ).ToList( );
    }



    public static class LeadStatusExtensions
    {

        public static bool IsLifeCycle( this LeadStatus status ) =>
            (int)status >= 101 && (int)status <= 125;

        public static bool IsUserProcessedOrLifeCycle( this LeadStatus status ) =>
            status == LeadStatus.UserProcessed || status.IsLifeCycle();

        public static bool IsValidNextLifeCycleStatus(this LeadStatus status) =>
            status.IsUserProcessedOrLifeCycle()
            || status == Contracts.LeadStatus.VIPRedial
            || status == Contracts.LeadStatus.FollowUpRedial;

        public static bool IsFirstProcessedTimeStatus(this LeadStatus status) =>
            status switch {
                LeadStatus.UserProcessed
                or LeadStatus.LifeCycleInitiation
                or LeadStatus.OrderConfirmationApproved
                or LeadStatus.ValidationApproved
                or LeadStatus.PaymentApproved
                    => true,
                _ => false
            };

        public static bool IsTriggerableStatus( this LeadStatus status )
        {
            switch( status )
            {
                case LeadStatus.New:
                case LeadStatus.RedialAutomatic:
                case LeadStatus.RedialManualCommon:
                case LeadStatus.RedialManualPrivate:
                case LeadStatus.VIPRedial:
                case LeadStatus.UserProcessed:
                case LeadStatus.Depleted:
                    return true;
            }

            return status.IsLifeCycle();
        }

        public static bool IsOpenPrivate( this LeadStatus status ) =>
            LeadStatuses.OpenPrivate.Contains( status );

        public static bool IsOpen( this LeadStatus status ) =>
            (int)status < 100;

        public static bool IsOpenOrLifeCycle( this LeadStatus status ) =>
            status.IsOpen() || status.IsLifeCycle();

        public static bool IsClosed( this LeadStatus status ) =>
            (int)status >= 200;

        public static bool IsRedial( this LeadStatus status ) =>
            status != LeadStatus.New && status.IsOpen();


        public static LeadStatus? ToStatus( this LeadImportStatus? importStatus ) =>
            !importStatus.HasValue
                ? (LeadStatus?)null
                : (LeadStatus)importStatus;
    }
}
